@@ -222,6 +222,9 @@ handler_move_end_cb (GtkWidget * h, gpointer data)
 
     /* TODO: adjust arrow direction based on which quarter of the screen we
      * are (Xinerama aware, of course ;) */
+
+    /* Need to save position here... */
+    write_panel_config ();
 }
 
 GtkWidget *
@@ -266,7 +269,7 @@ panel_set_hidden (Panel * p, gboolean hide)
     pos = p->position;
 
     /* xinerama aware screen coordinates */
-    if (pos.x < minx || pos.x > maxx || pos.y < miny || pos.y > maxy)
+    if (pos.x < minx || pos.x >= maxx || pos.y < miny || pos.y >= maxy)
     {
 	minx = MyDisplayX (pos.x, pos.y);
 	maxx = MyDisplayMaxX (dpy, scr, pos.x, pos.y);
@@ -277,6 +280,11 @@ panel_set_hidden (Panel * p, gboolean hide)
 	centery = miny + (maxy - miny) / 2;
     }
 
+    DBG("\n\tCoordinates:\n\t  min x, max x, min y, max y: %d, %d, %d, %d", 
+	    minx,maxx,miny,maxy);
+    DBG("\n\tPanel position (x,y): %d, %d", pos.x, pos.y);
+    DBG("\n\tPanel size: %d x %d\n", req.width, req.height);
+    
     /* Handle the resize */
     if (hide)
     {
@@ -312,7 +320,19 @@ panel_set_hidden (Panel * p, gboolean hide)
 	y = pos.y;
     }
 
-/*    g_print("(x=%d,y=%d) => (x=%d,y=%d)\n", pos.x, pos.y, x, y);*/
+    /* keep inside the screen */
+    if (x < 0)
+	x = 0;
+    else if (x > maxx - HIDDEN_SIZE)
+	x = maxx - HIDDEN_SIZE;
+
+    if (y < 0)
+	y = 0;
+    else if (y > maxy - HIDDEN_SIZE)
+	y = maxy - HIDDEN_SIZE;
+    
+    DBG("\n\tMove panel: (x=%d,y=%d) => (x=%d,y=%d)\n", pos.x, pos.y, x, y);
+
     if (hide)
 	gtk_widget_hide (p->main_frame);
     else
@@ -557,6 +577,7 @@ create_panel (void)
 {
     static gboolean need_init = TRUE;
     int x, y;
+    int hidden;
 
     /* necessary for initial settings to do the right thing */
     panel_created = FALSE;
@@ -589,15 +610,19 @@ create_panel (void)
 	need_init = FALSE;
     }
 
-    /* panel framework */
-    create_panel_framework (&panel);
-
+    /* unhide panel initially */
+    hidden = settings.autohide;
+    settings.autohide=FALSE;
+    
     /* FIXME
      * somehow the position gets set differently in the code below
      * we just save it here and restore it before reading the config
      * file */
     x = panel.position.x;
     y = panel.position.y;
+    
+    /* panel framework */
+    create_panel_framework (&panel);
 
     groups_init (GTK_BOX (panel.group_box));
 
@@ -617,7 +642,12 @@ create_panel (void)
      * otherwise the initial position will be messed up */
     panel_created = TRUE;
 
-    panel_set_autohide (settings.autohide);
+    panel.position.x = x;
+    panel.position.y = y;
+    panel_set_position ();
+
+    if (hidden)
+	panel_set_autohide(TRUE);
 }
 
 void
@@ -633,6 +663,7 @@ void
 panel_set_orientation (int orientation)
 {
     gboolean hidden;
+    int pos;
 
     settings.orientation = orientation;
 
@@ -645,11 +676,33 @@ panel_set_orientation (int orientation)
     if (hidden)
     {
 	panel_set_autohide (FALSE);
-
-	while (gtk_events_pending ())
-	    gtk_main_iteration ();
     }
 
+    gtk_widget_hide(panel.toplevel);
+    
+    /* change popup position to make it look better 
+     * done here, because it's needed for size calculation 
+     * the setttings dialog will also change, we just 
+     * anticipate that ;-)
+     */
+    pos = settings.popup_position;
+    switch (pos)
+    {
+	case LEFT:
+	    pos = TOP;
+	    break;
+	case RIGHT:
+	    pos = BOTTOM;
+	    break;
+	case TOP:
+	    pos = LEFT;
+	    break;
+	case BOTTOM:
+	    pos = RIGHT;
+	    break;
+    }
+    panel_set_popup_position(pos);
+
     /* save panel controls */
     groups_unpack ();
 
@@ -661,10 +714,17 @@ panel_set_orientation (int orientation)
     groups_set_orientation (orientation);
 
     panel.position.x = panel.position.y = -1;
-    panel_set_position ();
+    /* also sets position */
+    panel_set_size (settings.size);
 
+    gtk_widget_show(panel.toplevel);
+    set_window_layer (panel.toplevel, settings.layer);
+    set_window_skip (panel.toplevel);
+    
     if (hidden)
-	panel_set_autohide (TRUE);
+	panel_set_autohide(TRUE);
+
+    DBG("done");
 }
 
 void
@@ -676,6 +736,9 @@ panel_set_layer (int layer)
 	return;
 
     set_window_layer (panel.toplevel, layer);
+    
+    if (layer == TOP)
+	gtk_window_present(GTK_WINDOW(panel.toplevel));
 }
 
 void
@@ -763,9 +826,6 @@ panel_center (int side)
     if (hidden)
     {
 	panel_set_autohide (FALSE);
-
-	while (gtk_events_pending ())
-	    gtk_main_iteration ();
     }
 
     gtk_widget_size_request (panel.toplevel, &req);
@@ -826,9 +886,6 @@ panel_set_position (void)
 	    DBG ("unhide panel before repositioning\n");
 
 	    panel_set_autohide (FALSE);
-
-	    while (gtk_events_pending ())
-		gtk_main_iteration ();
 	}
 
 	w = screen_width;
@@ -853,6 +910,9 @@ panel_set_position (void)
 	gtk_window_move (GTK_WINDOW (panel.toplevel), panel.position.x,
 			 panel.position.y);
 
+        /* Need to save position here... */
+        write_panel_config ();
+
 	if (hidden)
 	    panel_set_autohide (TRUE);
     }
@@ -866,7 +926,18 @@ panel_set_autohide (gboolean hide)
     if (!panel_created)
 	return;
 
-    panel_set_hidden (&panel, hide);
+    if (hide)
+    {
+	DBG("add hide timeout");
+
+	panel.hide_timeout = g_timeout_add (1000, 
+					    (GSourceFunc) panel_hide_timeout, 
+					    &panel);
+    }
+    else
+    {
+	panel_set_hidden (&panel, hide);
+    }
 }
 
 /*  Global preferences
@@ -943,6 +1014,7 @@ panel_parse_xml (xmlNodePtr node)
 
     if (value)
     {
+	DBG ("number of groups: %s", value);
 	settings.num_groups = atoi (value);
 	g_free (value);
     }
@@ -1022,8 +1094,8 @@ panel_parse_xml (xmlNodePtr node)
 	settings.orientation = HORIZONTAL;
     if (settings.size < TINY || settings.size > LARGE)
 	settings.size = SMALL;
-    if (settings.num_groups < 1 || settings.num_groups > 2 * NBGROUPS)
-	settings.num_groups = 10;
+/*    if (settings.num_groups < 1 || settings.num_groups > 2 * NBGROUPS)
+	settings.num_groups = 2*NBGROUPS; */
 }
 
 void
